TOTALPOWER = "Total power"
HOUSEPOWER = "House power"
CARPOWERSENSOR = "CarPower-sensor"
CURRENTPEAKSENSOR = "CurrentPeak-sensor"

AVERAGECONSUMPTION = "Average consumption"
AVERAGECONSUMPTION_24H = "Average consumption 24h"
CONSUMPTION_TOTAL_NAME = "Energy including car"
CHARGERENABLED = "Charger enabled"
CHARGERDONE = "Charger done"